declare
  counter                  numeric(16);
  conn_id                  numeric(16);
  conn_name                varchar(64) := 'CSV - Target System assignments sync (Full)';
  conn_source              numeric(16) := 1; -- IDEAS driver
  conn_channel_mode        numeric(16) := 3; -- 0: Both, 1: Read from, 2: Write to, 3: Recon

  drv_type_id              numeric(16);
  drv_type_name            varchar(64) := 'CSV';
  
  cname_id                 numeric(16);
  cname_name               varchar(256) := 'SimpleCsv';
  cname_value              varchar(256);
  
  drv_id                   numeric(16);
  drv_name                 varchar(64) := 'CSVDriver4';
  
  drv_prop_id              numeric(16);
  
  conn_prop_id             numeric(16);

  dest_obj_class_id        numeric(16);
  dest_obj_class_name      varchar(64) := 'ACCOUNT';

  src_obj_class_id         numeric(16);
  src_obj_class_name       varchar(64) := 'CSVDriver';

  dest_obj_class_field_id  numeric(16);
  
  src_obj_class_field_id   numeric(16);
  
  mapping_obj_class_id     numeric(16);

  mapping_id               numeric(16);
  
  mapping_src_field_id     numeric(16);

begin

  -- Checking connector existence: if it already exists, exit
  select count(1) into counter from connector c where lower ( c.name ) = lower ( conn_name );
  if ( counter > 0 ) then
    RAISE_APPLICATION_ERROR ( -20200, 'CONNECTOR ALREADY EXISTS!');
  end if;

  -- Retrieving the destination obj class ID given the name (--> dest_obj_class_id). If it does not exist, exit
  select count(1) into counter from obj_class oc where lower ( oc.name ) = lower ( dest_obj_class_name ) and oc.driver = 1;
  if ( counter = 1 ) then
    select oc.id into dest_obj_class_id from obj_class oc where lower ( oc.name ) = lower ( dest_obj_class_name ) and oc.driver = 1;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FOUND!');
  end if;
  
  -- Checking driver type existence: if it does not exist, exit
  select count(1) into counter from driver_type dt where lower ( dt.name ) = lower ( drv_type_name );
  if ( counter = 1 ) then
    select dt.id into drv_type_id from driver_type dt where lower ( dt.name ) = lower ( drv_type_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DRIVER TYPE FOUND!');
  end if;
  
  -- Retrieving the class name ID and value given the name and driver ID (--> <cname_id,cname_value>)
  select count(1) into counter from class_name cn where lower ( cn.name ) = lower ( cname_name ) and cn.type = drv_type_id;
  if ( counter = 1 ) then
    select cn.id into cname_id from class_name cn where lower ( cn.name ) = lower ( cname_name ) and cn.type = drv_type_id;
    select cn.value into cname_value from class_name cn where lower ( cn.name ) = lower ( cname_name ) and cn.type = drv_type_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO CLASS NAME FOUND!');
  end if;
  
  -- Creating new DRIVER
  select driver_seq.nextval into drv_id from dual;
  insert into driver ( id, name, class, driver_type, state, multi_target )
  values ( drv_id, drv_name, cname_value, drv_type_id, 1, 1 );
  
  -- Creating new DRIVER_PROPERTY
  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'attributesColumnsName', 'USERID;EMAIL;DISABLED;AUTHORIZATION;AUTH_TYPE', 'attributesColumnsName', null, null, '1', null, null, null, 1, null, 'Names of the columns of the CSV divided by separator ( Ex. ID,NAME,GIVENNAME ). Set it to EXTERNALFILE, to use the file descriptor.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY (id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'csvSeparator', ';', 'csvSeparator', null, null, '2', null, null, null, 1, null, 'Enter the symbol representing the separator' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'pathFile', '/userdata/connectors/csv/target/assignments/', 'pathFile', null, null, '3', null, null, null, 1, null, 'Specify the directory name from which the CSV file will be read.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'attributesColumnsNameFileDescriptor', null, 'attributesColumnsNameFileDescriptor', null, null, '4', null, null, null, 0, null, 'Use this file to specify the CSV header.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'writeMode', null, 'writeMode', null, null, null, null, null, null, 0, null, 'Specify the write mode as ''append'', to modify the same file with each event. Specify the write mode as ''write'', to create a new file for each event.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'howToWriteColumns', null, 'howToWriteColumns', null, null, null, null, null, null, 0, null, 'Choose the columns that will appear in the output file.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'formatFileName', null, 'formatFileName', null, null, null, null, null, null, 0, null, 'Use this rule to name the output file.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'PathForEvents', null, 'PathForEvents', null, null, null, null, null, null, 0, null, 'Use this file to log the events.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'ignoreFirstLine', 'true', 'ignoreFirstLine', null, '2', null, null, null, null, 0, null, 'If the CSV file contains a header such as "ID;NAME;GIVENNAME", set "true". Otherwise, set "false".' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'csvOrder', '1', 'csvOrder', null, null, null, null, null, null, 0, null, 'Specify the filter by which the CSV file order will be sorted. By default, the CSV file order will be sorted by file name. Enter ''1'' to sort by file name, ''2'' to sort by size, ''3'' to sort by date, and ''4'' to sort by type. When you choose to display your results in descending order, enter the same numbers in negative. For e.g. -1 to sort by file name in descending order.' );

  select driver_property_seq.nextval into drv_prop_id from dual;
  insert into DRIVER_PROPERTY ( id, driver, name, value, displayed_name, state, attr1, attr2, attr3, attr4, attr5, mandatory, type, description )
  values ( drv_prop_id, drv_id, 'ATTRIBUTE_NAME_TO_DENORMALIZE', null, 'Attribute to denomarlize', null, null, null, null, null, null, 0, null, 'Attribute name to be used for the target events de-normalization. If it is not set, there will be no de-normalization.' );

  -- Creating new CONNECTOR
  select connector_seq.nextval into conn_id from dual;
  insert into connector ( id, name, destination, source, channel_mode )
  values ( conn_id, conn_name, drv_id, conn_source, conn_channel_mode );

  -- Creating new CONNECTOR_PROPERTY
  select connector_property_seq.nextval into conn_prop_id from dual;
  insert into CONNECTOR_PROPERTY (id, connector, name, value)
  values (conn_prop_id, conn_id, 'auditEnabled', 'true');
  
  select connector_property_seq.nextval into conn_prop_id from dual;
  insert into CONNECTOR_PROPERTY (id, connector, name, value)
  values (conn_prop_id, conn_id, 'modifyToAdd', 'false');
  
  select connector_property_seq.nextval into conn_prop_id from dual;
  insert into CONNECTOR_PROPERTY (id, connector, name, value)
  values (conn_prop_id, conn_id, 'reconciliationCode', '35');
  
  -- Creating new OBJECT_CLASS (source obj class)
  select obj_class_seq.nextval into src_obj_class_id from dual;
  insert into obj_class ( id, name, driver )
  values ( src_obj_class_id, src_obj_class_name, drv_id );

  -- New MAPPING_OBJ_CLASS
  select mapping_obj_class_seq.nextval into mapping_obj_class_id from dual;
  insert into mapping_obj_class ( id, connector, destination_obj_class, source_obj_class, channel_mode )
  values ( mapping_obj_class_id, conn_id, dest_obj_class_id, src_obj_class_id, conn_channel_mode );

  /*
   * Mapping creation:
   * new element in tables MAPPING, OBJECT_CLASS_FIELD, MAPPING_SOURCE_FIELD for each of the following mapping:
   *
   * USERID        --> CODE
   * AUTH_TYPE     --> TYPE (PERMISSION.TYPE)
   * DISABLED      --> DISABLED
   * EMAIL         --> EMAIL
   * AUTHORIZATION --> NAME (PERMISSION.NAME)
   *
  */

  select count(1) into counter from obj_class_field ocf where lower ( ocf.name ) = lower ( 'CODE' ) and obj_class = dest_obj_class_id;
  if ( counter = 1 ) then
    select ocf.id into dest_obj_class_field_id from obj_class_field ocf where lower ( ocf.name ) = lower ( 'CODE' ) and obj_class = dest_obj_class_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FIELD FOUND!');
  end if;
  select mapping_seq.nextval into mapping_id from dual;
  insert into mapping ( id, destination_field, function, key, channel_mode, mapping_obj_class )
  values ( mapping_id, dest_obj_class_field_id, 1, 1, conn_channel_mode, mapping_obj_class_id );
  select obj_class_field_seq.nextval into src_obj_class_field_id from dual;
  insert into obj_class_field ( id, name, type, obj_class )
  values ( src_obj_class_field_id, 'USERID', 'java.lang.String', src_obj_class_id );
  select mapping_source_field_seq.nextval into mapping_src_field_id from dual;
  insert into mapping_source_field ( id, mapping, source_field, fixed_value )
  values ( mapping_src_field_id, mapping_id, src_obj_class_field_id, 0 );

  select count(1) into counter from obj_class_field ocf where lower ( ocf.name ) = lower ( 'TYPE' ) and obj_class = dest_obj_class_id;
  if ( counter = 1 ) then
    select ocf.id into dest_obj_class_field_id from obj_class_field ocf where lower ( ocf.name ) = lower ( 'TYPE' ) and obj_class = dest_obj_class_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FIELD FOUND!');
  end if;
  select mapping_seq.nextval into mapping_id from dual;
  insert into mapping ( id, destination_field, function, key, channel_mode, mapping_obj_class )
  values ( mapping_id, dest_obj_class_field_id, 1, 0, conn_channel_mode, mapping_obj_class_id );
  select obj_class_field_seq.nextval into src_obj_class_field_id from dual;
  insert into obj_class_field ( id, name, type, obj_class )
  values ( src_obj_class_field_id, 'AUTH_TYPE', 'java.lang.String', src_obj_class_id );
  select mapping_source_field_seq.nextval into mapping_src_field_id from dual;
  insert into mapping_source_field ( id, mapping, source_field, fixed_value )
  values ( mapping_src_field_id, mapping_id, src_obj_class_field_id, 0 );

  select count(1) into counter from obj_class_field ocf where lower ( ocf.name ) = lower ( 'DISABLED' ) and obj_class = dest_obj_class_id;
  if ( counter = 1 ) then
    select ocf.id into dest_obj_class_field_id from obj_class_field ocf where lower ( ocf.name ) = lower ( 'DISABLED' ) and obj_class = dest_obj_class_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FIELD FOUND!');
  end if;
  select mapping_seq.nextval into mapping_id from dual;
  insert into mapping ( id, destination_field, function, key, channel_mode, mapping_obj_class )
  values ( mapping_id, dest_obj_class_field_id, 1, 0, conn_channel_mode, mapping_obj_class_id );
  select obj_class_field_seq.nextval into src_obj_class_field_id from dual;
  insert into obj_class_field ( id, name, type, obj_class )
  values ( src_obj_class_field_id, 'DISABLED', 'java.lang.String', src_obj_class_id );
  select mapping_source_field_seq.nextval into mapping_src_field_id from dual;
  insert into mapping_source_field ( id, mapping, source_field, fixed_value )
  values ( mapping_src_field_id, mapping_id, src_obj_class_field_id, 0 );

  select count(1) into counter from obj_class_field ocf where lower ( ocf.name ) = lower ( 'EMAIL' ) and obj_class = dest_obj_class_id;
  if ( counter = 1 ) then
    select ocf.id into dest_obj_class_field_id from obj_class_field ocf where lower ( ocf.name ) = lower ( 'EMAIL' ) and obj_class = dest_obj_class_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FIELD FOUND!');
  end if;
  select mapping_seq.nextval into mapping_id from dual;
  insert into mapping ( id, destination_field, function, key, channel_mode, mapping_obj_class )
  values ( mapping_id, dest_obj_class_field_id, 1, 0, conn_channel_mode, mapping_obj_class_id );
  select obj_class_field_seq.nextval into src_obj_class_field_id from dual;
  insert into obj_class_field ( id, name, type, obj_class )
  values ( src_obj_class_field_id, 'EMAIL', 'java.lang.String', src_obj_class_id );
  select mapping_source_field_seq.nextval into mapping_src_field_id from dual;
  insert into mapping_source_field ( id, mapping, source_field, fixed_value )
  values ( mapping_src_field_id, mapping_id, src_obj_class_field_id, 0 );

  select count(1) into counter from obj_class_field ocf where lower ( ocf.name ) = lower ( 'NAME' ) and obj_class = dest_obj_class_id;
  if ( counter = 1 ) then
    select ocf.id into dest_obj_class_field_id from obj_class_field ocf where lower ( ocf.name ) = lower ( 'NAME' ) and obj_class = dest_obj_class_id;
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO DEST OBJ CLASS FIELD FOUND!');
  end if;
  select mapping_seq.nextval into mapping_id from dual;
  insert into mapping ( id, destination_field, function, key, channel_mode, mapping_obj_class )
  values ( mapping_id, dest_obj_class_field_id, 1, 0, conn_channel_mode, mapping_obj_class_id );
  select obj_class_field_seq.nextval into src_obj_class_field_id from dual;
  insert into obj_class_field ( id, name, type, obj_class )
  values ( src_obj_class_field_id, 'AUTHORIZATION', 'java.lang.String', src_obj_class_id );
  select mapping_source_field_seq.nextval into mapping_src_field_id from dual;
  insert into mapping_source_field ( id, mapping, source_field, fixed_value )
  values ( mapping_src_field_id, mapping_id, src_obj_class_field_id, 0 );

end;
/
commit;

