declare
  rq_id            numeric(16);
  rq_name          varchar(256) := 'Certification - Fulfilment status report';
  rq_desc          varchar(512) := 'Status summary with details around the fulfilment status, (applicable for the application not subject to write back sync)';
  rqd              clob;
  counter          numeric(16);

begin
  rqd := 'select distinct t.CAMPAIGN_NAME as CAMPAIGN_NAME,
'||'       t.CAMPAIGN_START as CAMPAIGN_START,
'||'       t.CAMPAIGN_END as CAMPAIGN_END,
'||'       t.CAMPAIGN_STATUS as CAMPAIGN_STATUS,
'||'       t.APPLICATION_NAME as APPLICATION_NAME,
'||'       t.PERMISSION_NAME as PERMISSION_NAME,
'||'       t.USER_CODE as USER_CODE,
'||'       t.USER_NAME as USER_NAME,
'||'       t.USER_SURNAME as USER_SURNAME,
'||'       t.OU_CODE as OU_CODE,
'||'       t.OU_NAME as OU_NAME,
'||'       t.ATTESTATION_STATUS as ATTESTATION_STATUS,
'||'       t.REVIEWED_BY_INFO REVIEWED_BY_INFO,
'||'       t.REVIEW_DATE as REVIEW_DATE,
'||'       t.REVOKE_FULFILLMENT as REVOKE_FULFILLMENT
'||'from
'||'(
'||'  select att.name as CAMPAIGN_NAME,
'||'         att.start_date as CAMPAIGN_START,
'||'         att.end_date as CAMPAIGN_END,
'||'         ''Active'' as CAMPAIGN_STATUS,
'||'         a.name as APPLICATION_NAME,
'||'         e.name as PERMISSION_NAME,
'||'         p.code as USER_CODE,
'||'         p.name as USER_NAME,
'||'         p.surname as USER_SURNAME,
'||'         ou.code as OU_CODE,
'||'         ou.name as OU_NAME,
'||'         case
'||'           when er.review_state = 0 then ''Not recertified yet''
'||'           when er.review_state = 1 then ''Approved''
'||'           when er.review_state = 2 or er.review_state = 3 then ''Revoked''
'||'           when er.review_state >= 10 then ''Other''
'||'         end as ATTESTATION_STATUS,
'||'         er.reviewed_by_info as REVIEWED_BY_INFO,
'||'         er.review_date as REVIEW_DATE,
'||'         case
'||'           when er.review_state in ( 0, 1 ) then null
'||'           when er.employment is not null then ''Pending''
'||'           else ''Removed''
'||'         end as REVOKE_FULFILLMENT,
'||'         att.state as campaign_status_num,
'||'         ou.id as ou_id,
'||'         a.id as app_id
'||'    from #pmschema#.attestation              att,
'||'         #pmschema#.employment_review        er,
'||'         #pmschema#.person                   p,
'||'         #pmschema#.organizational_unit      ou,
'||'         #pmschema#.entitlement              e,
'||'         #pmschema#.entitlement_flat_hier  efh,
'||'         #pmschema#.application              a
'||'   where att.id = er.attestation
'||'     and att.type = 1
'||'     and er.person = p.id
'||'     and p.organizational_unit = ou.id
'||'     and er.entitlement = e.id
'||'     and e.id = efh.parent
'||'     and efh.child_application = a.id
'||'     and efh.child_int_type = 1
'||'     and ou.hierarchy = 1
'||'	 and not exists
'||'		(select 1 from #pmschema#.employment_review_h erh 
'||'		where erh.entitlement_name=e.name
'||'		and erh.application_name=a.name
'||'		and erh.attestation_name=att.name 
'||'		and erh.person=er.person)
'||'  union
'||'  select erh.attestation_name as CAMPAIGN_NAME,
'||'         null as CAMPAIGN_START,
'||'         null as CAMPAIGN_END,
'||'         ''Inactive'' as CAMPAIGN_STATUS,
'||'         a.name as APPLICATION_NAME,
'||'         e.name as PERMISSION_NAME,
'||'         p.code as USER_CODE,
'||'         p.name as USER_NAME,
'||'         p.surname as USER_SURNAME,
'||'         ou.code as OU_CODE,
'||'         ou.name as OU_NAME,
'||'         case
'||'           when erh.review_state = 0 then ''Not recertified''
'||'           when erh.review_state = 1 then ''Approved''
'||'           when erh.review_state = 2 or erh.review_state = 3 then ''Revoked''
'||'           when erh.review_state >= 10 then ''Other''
'||'         end as ATTESTATION_STATUS,
'||'         erh.cert_first_owner_name || '' '' || erh.cert_first_owner_surname || '' ('' || erh.cert_first_owner_code || '')'' as REVIEWED_BY_INFO,
'||'         erh.review_date as REVIEW_DATE,
'||'         case
'||'           when erh.review_state in ( 0, 1 ) then null
'||'           --when erh.employment is not null then ''Pending''
'||'           else ''Removed''
'||'         end as REVOKE_FULFILLMENT,
'||'         null as campaign_status_num,
'||'         ou.id as ou_id,
'||'         a.id as app_id
'||'    from #pmschema#.employment_review_h       erh,
'||'         #pmschema#.person                    p,
'||'         #pmschema#.organizational_unit       ou,
'||'         #pmschema#.entitlement               e,
'||'         #pmschema#.entitlement_flat_hier     efh,
'||'         #pmschema#.application               a
'||'   where erh.person = p.id
'||'     and p.organizational_unit = ou.id
'||'     and erh.entitlement = e.id
'||'     and e.id = efh.parent
'||'     and efh.child_application = a.id
'||'     and efh.child_int_type = 1
'||'     and ou.hierarchy = 1
'||'  union
'||'  select er.attestation_name as CAMPAIGN_NAME,
'||'         null as CAMPAIGN_START,
'||'         null as CAMPAIGN_END,
'||'         ''Inactive'' as CAMPAIGN_STATUS,
'||'         ''N/A'' as APPLICATION_NAME,
'||'         er.entitlement_name as PERMISSION_NAME,
'||'         p.code as USER_CODE,
'||'         p.name as USER_NAME,
'||'         p.surname as USER_SURNAME,
'||'         ou.code as OU_CODE,
'||'         ou.name as OU_NAME,
'||'         case
'||'           when er.review_state = 0 then ''Not recertified''
'||'           when er.review_state = 1 then ''Approved''
'||'           when er.review_state = 2 or er.review_state = 3 then ''Revoked''
'||'           when er.review_state >= 10 then ''Other''
'||'         end as ATTESTATION_STATUS,
'||'         /*er.reviewed_by_name || '' '' || er.reviewed_by_surname || '' ('' || er.reviewed_by_code || '')''*/ null as REVIEWED_BY_INFO,
'||'         er.review_date as REVIEW_DATE,
'||'         ''System deleted'' as REVOKE_FULFILLMENT,
'||'         null as campaign_status_num,
'||'         ou.id as ou_id,
'||'         null as app_id
'||'    from #pmschema#.employment_review_h er,
'||'         #pmschema#.person                    p,
'||'         #pmschema#.organizational_unit       ou
'||'   where er.person = p.id
'||'     and p.organizational_unit = ou.id
'||'     and er.entitlement is null
'||'     and ou.hierarchy = 1
'||') t,
'||'  #schema_tmp#.tmp_rep_application tmp1,
'||'  #schema_tmp#.tmp_rep_organizational_unit tmp2
'||'where
'||'  lower(t.attestation_status) = lower(''#attestation_status#'')
'||'  and lower(t.revoke_fulfillment) = lower(''#revoke_fulfillment#'')
'||'  and (tmp1.id = t.app_id or t.app_id is null)
'||'  and tmp2.id = t.ou_id
';

  select count(1) into counter from r_query t where lower ( t.name ) = lower ( rq_name );
  if ( counter = 1 ) then
    select t.id into rq_id from r_query t where lower ( t.name ) = lower ( rq_name );

    update r_query_data set query = rqd where r_query = rq_id;
  end if;
  
end;
/
commit;