declare
  counter       number(16);
  conn_id       number(16);
  conn_name     varchar2(64) := 'CSV - HR Feed Users  (Delta)';
  
  im_id         number(16);
  im_code       varchar2(256) := 'EC';

  fc_id         number(16);
  fc_name       varchar2(256) := 'advanced';

  fet_id        number(16);
  fet_name      varchar2(256);

  f_id          number(16);
  f_name        varchar2(256);

  f_lob         clob := 'package ec 
'||'
'||'import com.crossideas.ideasconnector.core.databean.Event;
'||'import com.crossideas.ideasconnector.core.databean.DataBean;
'||'import java.util.List;
'||'import java.util.ArrayList;
'||'import java.util.Map;
'||'import java.util.HashMap;
'||'
'||'global com.crossideas.ideasconnector.drivers.interfaces.Driver srcDriver;
'||'global com.crossideas.ideasconnector.drivers.interfaces.Driver destDriver;
'||'global com.crossideas.ideasconnector.common.xml.configuration.TargetDriver config;
'||'global com.engiweb.logger.ILogger log;
'||'
';

  fe_id         number(16);
  fer_id        number(16);

  fr_id         number(16);
  fr_name       varchar2(256) := 'Manage ACTION';
  fr_desc       varchar2(512) := 'Change the event type according to ACTION';
  
  fr_lob        clob := 'rule "Manage ACTION"
'||'when
'||'    event : Event(  )  
'||'then
'||'//
'||'        String action = (String) event.getBean().getCurrentAttribute("ACTION");
'||'
'||'        if (action == null) {
'||'          // ADD
'||'          return;
'||'        }
'||'
'||'        if (action.equals("MOD")) {
'||'            
'||'            event.setType(Event.TYPE_MODIFY);
'||'        } 
'||'
'||'        if (action.equals("DEL")) {
'||'            
'||'            event.setType(Event.TYPE_DELETE);
'||'        } 
'||'
'||'end
'||'';

begin

  -- Retrieving ideas module ID given the code (--> im_id)
  select count(1) into counter from ideas_module t where lower ( t.code ) = lower ( im_code );
  if ( counter = 1 ) then
    select t.id into im_id from ideas_module t where lower ( t.code ) = lower ( im_code );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO IDEAS MODULE FOUND' );
  end if;

  -- Retrieving flow context ID given the name (--> fc_id)
  select count(1) into counter from flow_context t where lower ( t.name ) = lower ( fc_name );
  if ( counter = 1 ) then
    select t.id into fc_id from flow_context t where lower ( t.name ) = lower ( fc_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW CONTEXT FOUND' );
  end if;

  -- Retrieving connector ID given the name (--> conn_id)
  select count(1) into counter from connector t where lower ( t.name ) = lower ( conn_name );
  if ( counter = 1 ) then
    select t.id into conn_id from connector t where lower ( t.name ) = lower ( conn_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO CONNECTOR FOUND');
  end if;
  
  /*
   * Creating flow_event_type ID_WRITETO and related flows
   */
  fet_name := conn_id || '_WRITETO';

  -- Creating flow_event_type if it does not already exist
  select count(1) into counter from flow_event_type t
  where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  if ( counter = 0 ) then
    select flow_event_type_seq.nextval into fet_id from dual;
	insert into flow_event_type ( id, flow_context, name, ideas_module )
	values ( fet_id, fc_id, fet_name, im_id );
  else
    select t.id into fet_id from flow_event_type t
    where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  end if;

  -- Creating flow ID_WRITETO_DATA_TRANSFORMATION if it does not already exist
  f_name := fet_name || '_DATA_TRANSFORMATION';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name );
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  -- Creating flow ID_WRITETO_POST_MAPPING if it does not already exist
  f_name := fet_name || '_POST_MAPPING';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name );
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  -- Creating flow ID_WRITETO_RESPONSE if it does not already exist
  f_name := fet_name || '_RESPONSE';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name );
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  /*
   * Creating flow_event_type ID_READFROM and related flows
   */
  fet_name := conn_id || '_READFROM';

  -- Creating flow_event_type if it does not already exist
  select count(1) into counter from flow_event_type t
  where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  if ( counter = 0 ) then
    select flow_event_type_seq.nextval into fet_id from dual;
	insert into flow_event_type ( id, flow_context, name, ideas_module )
	values ( fet_id, fc_id, fet_name, im_id );
  else
    select t.id into fet_id from flow_event_type t
    where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  end if;

  -- Creating flow ID_READFROM_DATA_TRANSFORMATION if it does not already exist
  f_name := fet_name || '_DATA_TRANSFORMATION';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  end if;

  -- Creating root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  else
    select t.id into fe_id from flow_entity t
	where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null;
  end if;

  -- Creating flow rule for the flow created
  select count(1) into counter from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );
  if ( counter = 0 ) then
    select flow_rule_seq.nextval into fr_id from dual;
    insert into flow_rule ( id, flow, name, description, rule_lob, ideas_module )
    values ( fr_id, f_id, fr_name, fr_desc, fr_lob, im_id );
  else
    select t.id into fr_id from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );
    update flow_rule set description = fr_desc where id = fr_id;
    update flow_rule set rule_lob = fr_lob where id = fr_id;
  end if;

  -- Creating rule flow entity type for the flow created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and flow_type = 2
  and t.flow_entity_type = 1 and t.root = fe_id and t.flow_rule = fr_id;
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fer_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, flow_rule, position, ideas_module)
    values ( fer_id, f_id, 2, fe_id, fe_id, 1, fr_id, 0, im_id );
  end if;

  -- Creating flow ID_READFROM_POST_MAPPING if it does not already exist
  f_name := fet_name || '_POST_MAPPING';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  /*
   * Creating flow_event_type ID_RECONCILIATION and related flows
   */
  fet_name := conn_id || '_RECONCILIATION';

  -- Creating flow_event_type if it does not already exist
  select count(1) into counter from flow_event_type t
  where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  if ( counter = 0 ) then
    select flow_event_type_seq.nextval into fet_id from dual;
	insert into flow_event_type ( id, flow_context, name, ideas_module )
	values ( fet_id, fc_id, fet_name, im_id );
  else
    select t.id into fet_id from flow_event_type t
    where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fet_name );
  end if;

  -- Creating flow ID_RECONCILIATION_DATA_TRANSFORMATION if it does not already exist
  f_name := fet_name || '_DATA_TRANSFORMATION';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  -- Creating flow ID_RECONCILIATION_POST_MAPPING if it does not already exist
  f_name := fet_name || '_POST_MAPPING';
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 0 ) then
    select flow_seq.nextval into f_id from dual;
	insert into flow ( id, flow_event_type, name, flow_lob, ideas_module )
	values ( f_id, fet_id, f_name, f_lob, im_id );
  else
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fet_id and lower ( t.name ) = lower ( f_name ); 
  end if;

  -- Create root flow entity type for the flow just created
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = 2 and t.flow_entity_type = 0 and t.parent is null; 
  if ( counter = 0 ) then
    select flow_entity_seq.nextval into fe_id from dual;
    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, position, ideas_module)
    values ( fe_id, f_id, 2, fe_id, null, 0, 0, im_id );
  end if;

  /*
  _WRITETO_DATA_TRANSFORMATION
  _WRITETO_POST_MAPPING
  _WRITETO_RESPONSE
  _READFROM_DATA_TRANSFORMATION
  _READFROM_POST_MAPPING
  _RECONCILIATION_DATA_TRANSFORMATION
  _RECONCILIATION_POST_MAPPING
  */
end;
/
commit;
