set define off
declare
  ft_id         number(16);
  ft_name       varchar2(256) := 'RUN';
  
  im_id         number(16);
  im_name       varchar2(256) := 'AGC';

  fc_id         number(16);
  fc_name       varchar2(256) := 'system';
  
  fet_root      number(16) := 0;
  fet_rule      number(16) := 1;
  fet_group     number(16) := 2;
  fet_split     number(16) := 3;
  fet_split_con number(16) := 4;
  
  fet_par_id    number(16);
  fet_par_type  number(16);
  
  fevt_id       number(16);
  fevt_name     varchar2(256) := 'IN';
  
  f_id          number(16);
  f_name        varchar2(256) := 'USER_ADD';
  
  fer_id        number(16);
  feg_id        number(16);
  feg_name      varchar2(256) := 'Add User default group';
  
  fe_pos        number(16);
  
  fr_id         number(16);
  fr_name       varchar2(256) := 'Create OrgUnit From User Data';
  fr_desc       varchar2(512) := '[ V1.4 - 2017-02-27 ] - Default Rule to create OU (if does not exists). The OU code is coming from User Data (column OU). By default the OUName is equal to OUCode.';
  
  fr_lob        clob := 'rule "Create OrgUnit From User Data"
'||'when
'||'    event : EventInBean(  )  
'||'    userErcBean : UserErcBean(  )  
'||'    userBean : UserBean(  )  
'||'    extInfoBean : ExternalInfo(  )  
'||'    orgUnitBean : OrgUnitBean(  )  
'||'then
'||'// User already exists
'||' if (userBean.getId() != null) {
'||'   return;
'||' }
'||'
'||' // Org Unit already exists
'||' if (orgUnitBean.getId() != null) {
'||'   return;
'||' }
'||'
'||' // User OU Name (considered unique)
'||' String userOU = (String) userErcBean.getAttribute("OU");
'||' if(userOU == null || userOU.trim().isEmpty()) {
'||'   // set OU as root OU
'||'   orgUnitBean.setId(1L);
'||'   return;
'||' }
'||' OrgUnitBean ouEmployee = new OrgUnitBean();
'||' ouEmployee.setName(userOU);
'||' ouEmployee.setCode(userOU);
'||'
'||'   // Root OU
'||' OrgUnitBean root = new OrgUnitBean();
'||' root.setId(1L);
'||' 
'||' // Create User OU under Root (Flat Hierarchy generation under root)
'||' ouEmployee = OrgUnitAction.add(sql, ouEmployee, root, true, true);
'||'
'||' logger.debug("User OU created under Root : " + ouEmployee);
'||'
'||' // Make ouEmployee available for the next rules
'||' orgUnitBean.setId(ouEmployee.getId()); 
'||'
'||'end';

  counter       number(16);
begin
  select count(1) into counter from flow_type t where lower ( t.name ) = lower ( ft_name );
  if ( counter = 1 ) then
    select t.id into ft_id from flow_type t where lower ( t.name ) = lower ( ft_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW TYPE FOUND' );
  end if;

  select count(1) into counter from ideas_module t where lower ( t.code ) = lower ( im_name );
  if ( counter = 1 ) then
    select t.id into im_id from ideas_module t where lower ( t.code ) = lower ( im_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO IDEAS MODULE FOUND' );
  end if;

  select count(1) into counter from flow_context t where lower ( t.name ) = lower ( fc_name );
  if ( counter = 1 ) then
    select t.id into fc_id from flow_context t where lower ( t.name ) = lower ( fc_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW CONTEXT FOUND' );
  end if;

  select count(1) into counter from flow_event_type t
  where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fevt_name );
  if ( counter = 1 ) then
    select t.id into fevt_id from flow_event_type t
    where t.flow_context = fc_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fevt_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW EVENT TYPE FOUND' );
  end if;
  
  select count(1) into counter from flow t
  where t.ideas_module = im_id and t.flow_event_type = fevt_id and lower ( t.name ) = lower ( f_name ); 
  if ( counter = 1 ) then
    select t.id into f_id from flow t
    where t.ideas_module = im_id and t.flow_event_type = fevt_id and lower ( t.name ) = lower ( f_name ); 
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO FLOW FOUND' );
  end if;

  select count(1) into counter from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );
  if ( counter = 0 ) then
    select flow_rule_seq.nextval into fr_id from dual;
    insert into flow_rule ( id, flow, name, description, rule_lob, ideas_module )
    values ( fr_id, f_id, fr_name, fr_desc, fr_lob, im_id );
  else
    select t.id into fr_id from flow_rule t where t.flow = f_id and t.ideas_module = im_id and lower ( t.name ) = lower ( fr_name );

    update flow_rule set description = fr_desc where id = fr_id;

    update flow_rule set rule_lob = fr_lob where id = fr_id;
  end if;

  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and t.flow_type = ft_id and t.flow_entity_type = fet_root and t.parent is null; 
  if ( counter = 1 ) then
    select t.id into fer_id from flow_entity t
    where t.ideas_module = im_id and t.flow = f_id and t.flow_type = ft_id and t.flow_entity_type = fet_root and t.parent is null; 
  end if;

  if ( feg_name is not null and length ( trim ( feg_name )  ) > 0 ) then
    select count(1) into counter from flow_entity t
    where t.ideas_module = im_id and t.flow = f_id and flow_type = ft_id and t.flow_entity_type = fet_group and t.root = fer_id;
    if ( counter = 0 ) then
      select flow_entity_seq.nextval into feg_id from dual;
      
      insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, name, position, ideas_module)
      values ( feg_id, f_id, ft_id, fer_id, fer_id, fet_group, feg_name, 0, im_id );
    else
      select t.id, t.flow_type into feg_id, fet_par_type from flow_entity t
      where t.ideas_module = im_id and t.flow = f_id and flow_type = ft_id and t.flow_entity_type = fet_group and t.root = fer_id;
    end if;
  end if;
  
  if ( feg_id is null ) then
    fet_par_id   := fer_id;
  else
    fet_par_id   := feg_id;
  end if;
  fet_par_type := fet_rule;
  
  select count(1) into counter from flow_entity t
  where t.ideas_module = im_id and t.flow = f_id and flow_type = ft_id
  and t.flow_entity_type = fet_par_type and t.root = fer_id and t.flow_rule = fr_id;
  if ( counter = 0 ) then
    select max ( t.position ) into fe_pos from flow_entity t
    where t.ideas_module = im_id and t.flow = f_id and flow_type = ft_id
    and t.flow_entity_type = fet_rule and t.root = fer_id;

    if ( fe_pos is null ) then
      fe_pos := 0;
    else
      fe_pos := fe_pos + 1;
    end if;

    insert into flow_entity ( id, flow, flow_type, root, parent, flow_entity_type, flow_rule, position, ideas_module)
    values ( flow_entity_seq.nextval, f_id, ft_id, fer_id, fet_par_id, fet_rule, fr_id, fe_pos, im_id );
  end if;

end;
/
commit;
set define on
