declare
  cur_realm         sys_refcursor;
  cur_class         sys_refcursor;
  cur_tw            sys_refcursor;

  realm_id          number(16);

  scheduler_id      number(16);
  scheduler_name    varchar2(256) := 'System';

  class_id          number(16);
  class_name        varchar2(256) := 'PollingDispatcherJob';
  class_value       varchar2(256) := 'com.engiweb.job.backend.scheduler.PollingDispatcherJob';
  class_desc        varchar2(512) := '';
  
  family_id         number(16);
  family_name       varchar2(256) := 'Ideas';

  timer_id          number(16);
  timer_name        varchar2(256) := 'Housekeeping';
  timer_desc        varchar2(512) := 'System task. High frequency recurring jobs collection -  data consistency related.';
  timer_repetition  number(16)    := null;
  timer_cron        varchar2(256) := '/30 * * * * ? *';
  timer_history     number(16)    := 0;

  work_id           number(16);
  work_name         varchar2(256) := 'DispatcherJob';
  work_desc         varchar2(512) := '';
  
  timer_work_id     number(16);
  timer_work_ord    number(16);--no need to set

  wp1_name          varchar2(256) := 'applicationName';
  wp1_value         varchar2(256) := 'AccessGovernanceCore';
  wp1_desc          varchar2(256) := '';
  wp1_type          number(16) := 0;
  wp1_mode_type     number(16) := 0;
  wp1_req           number(16) := 1;

  counter           number(16);
begin
  select count(1) into counter from scheduler t where lower ( t.name ) = lower ( scheduler_name );
  if ( counter = 1) then
    select t.id into scheduler_id from scheduler t where lower ( t.name ) = lower ( scheduler_name );
  else
    RAISE_APPLICATION_ERROR ( -20200, 'NO SCHEDULER FOUND!');
  end if;

  open cur_realm for
    select t.id from realm t;
  loop
    fetch cur_realm into realm_id;
    exit when cur_realm%NOTFOUND;

    select count(1) into counter from class_name t where lower ( t.name ) = lower ( class_name ) and lower ( t.value ) = lower ( class_value ) and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into class_id from class_name t where lower ( t.name ) = lower ( class_name ) and lower ( t.value ) = lower ( class_value ) and t.realm = realm_id;
    else
      select class_name_seq.nextval into class_id from dual;
      insert into class_name ( id, realm, name, value, description )
      values ( class_id, realm_id, class_name, class_value, class_desc );
    end if;

    select count(1) into counter from family t where lower ( t.name ) = lower ( family_name ) and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into family_id from family t where lower ( t.name ) = lower ( family_name ) and t.realm = realm_id;
    else
      select family_seq.nextval into family_id from dual;
      insert into family ( id, realm, name )
      values ( family_id, realm_id, family_name );
    end if;

    select count(1) into counter from timer t where lower ( t.name ) = lower ( timer_name ) and t.realm = realm_id and t.scheduler = scheduler_id;
    if ( counter = 1) then
      select t.id into timer_id from timer t where lower ( t.name ) = lower ( timer_name ) and t.realm = realm_id and t.scheduler = scheduler_id;
    else
      select timer_seq.nextval into timer_id from dual;
      insert into timer ( id, realm, name, description, family, repetition, cron_string, scheduler, history )
      values ( timer_id, realm_id, timer_name, timer_desc, family_id, timer_repetition, timer_cron, scheduler_id, timer_history );
    end if;

    select count(1) into counter from work t where lower ( t.name ) = lower ( work_name ) and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into work_id from work t where lower ( t.name ) = lower ( work_name ) and t.realm = realm_id;
    else
      select work_seq.nextval into work_id from dual;
      insert into work ( id, realm, name, description, class_name )
      values ( work_id, realm_id, work_name, work_desc, class_id );
    end if;

    select count(1) into counter from timer_work t where t.timer = timer_id and t.work = work_id and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into timer_work_id from timer_work t where t.timer = timer_id and t.work = work_id and t.realm = realm_id;
    else
      select timer_work_seq.nextval into timer_work_id from dual;

      select max ( t.seq_order ) into timer_work_ord from timer_work t where t.timer = timer_id and t.realm = realm_id;
      if ( timer_work_ord is null ) then
        timer_work_ord := 1;
      else
        timer_work_ord := timer_work_ord + 1;
      end if;
      
      insert into timer_work ( id, realm, timer, work, seq_order )
      values ( timer_work_id, realm_id, timer_id, work_id, timer_work_ord );
    end if;

    if ( length ( wp1_name ) > 0 ) then
      select count(1) into counter from work_prop t where lower ( t.name ) = lower ( wp1_name ) and t.realm = realm_id and t.work = work_id;
      if ( counter = 0 ) then
        insert into work_prop ( id, realm, work, name, mode_type, type, value, required, description )
        values ( work_prop_seq.nextval, realm_id, work_id, wp1_name, wp1_mode_type, wp1_type, wp1_value, wp1_req, wp1_desc );
      end if;

      select count(1) into counter from timer_work_prop t where lower ( t.name ) = lower ( wp1_name ) and t.realm = realm_id and t.timer_work = timer_work_id;
      if ( counter = 0 ) then
        insert into timer_work_prop ( id, realm, timer_work, name, mode_type, type, value, required, description )
        values ( timer_work_prop_seq.nextval, realm_id, timer_work_id, wp1_name, wp1_mode_type, wp1_type, wp1_value, wp1_req, wp1_desc );
      end if;
    end if;

  end loop;
  close cur_realm;

end;
/
commit;