declare
  cur_realm         sys_refcursor;

  realm_id          number(16);

  class_id          number(16);
  class_name        varchar2(256) := 'ArcsSingleOperationJob';
  class_value       varchar2(256) := 'com.engiweb.ccs.backend.job.Work';
  class_desc        varchar2(512) := '';
  
  work_id           number(16);
  work_name         varchar2(256) := 'ARCSSingleOperation';
  work_desc         varchar2(512) := 'Any task created using this job must specify the analysis type and the sap system name. The environment name is optional and can be specified only for the SoD analysis. The default environment is the working environment. Analysis type: -0- Sap full load, -1- Sap delta load, -2- Sap role analysis, -3- Collective role analysis, -4- User analysis, -5- SoD analysis, -6- Warning analysis';
  
  wp1_name          varchar2(256) := 'environmentName';
  wp1_value         varchar2(256) := '';
  wp1_desc          varchar2(256) := 'If not assigned use current working environment';
  wp1_type          number(16) := 0;
  wp1_mode_type     number(16) := 0;
  wp1_req           number(16) := 0;
  
  wp2_name          varchar2(256) := 'sapSystemName';
  wp2_value         varchar2(256) := '';
  wp2_desc          varchar2(256) := 'Sap system name';
  wp2_type          number(16) := 0;
  wp2_mode_type     number(16) := 0;
  wp2_req           number(16) := 1;
  
  wp3_name          varchar2(256) := 'typeAnalysis';
  wp3_value         varchar2(256) := '';
  wp3_desc          varchar2(256) := '-0- Sap full load, -1- Sap delta load, -2- Sap role analysis, -3- Collective role analysis, -4- User analysis, -5- SoD analysis, -6- Warning analysis';
  wp3_type          number(16) := 3;
  wp3_mode_type     number(16) := 0;
  wp3_req           number(16) := 1;

  counter           number(16);
begin
  open cur_realm for
    select t.id from realm t;
  loop
    fetch cur_realm into realm_id;
    exit when cur_realm%NOTFOUND;

    select count(1) into counter from class_name t where lower ( t.name ) = lower ( class_name ) and lower ( t.value ) = lower ( class_value ) and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into class_id from class_name t where lower ( t.name ) = lower ( class_name ) and lower ( t.value ) = lower ( class_value ) and t.realm = realm_id;
    else
      select class_name_seq.nextval into class_id from dual;
      insert into class_name ( id, realm, name, value, description )
      values ( class_id, realm_id, class_name, class_value, class_desc );
    end if;

    select count(1) into counter from work t where lower ( t.name ) = lower ( work_name ) and t.realm = realm_id;
    if ( counter = 1) then
      select t.id into work_id from work t where lower ( t.name ) = lower ( work_name ) and t.realm = realm_id;
	else  
      select work_seq.nextval into work_id from dual;
      insert into work ( id, realm, name, description, class_name )
      values ( work_id, realm_id, work_name, work_desc, class_id );
    end if;

    if ( length ( wp1_name ) > 0 ) then
      select count(1) into counter from work_prop t where lower ( t.name ) = lower ( wp1_name ) and t.realm = realm_id and t.work = work_id;
      if ( counter = 0 ) then
        insert into work_prop ( id, realm, work, name, mode_type, type, value, required, description )
        values ( work_prop_seq.nextval, realm_id, work_id, wp1_name, wp1_mode_type, wp1_type, wp1_value, wp1_req, wp1_desc );
      end if;
    end if;

    if ( length ( wp2_name ) > 0 ) then
      select count(1) into counter from work_prop t where lower ( t.name ) = lower ( wp2_name ) and t.realm = realm_id and t.work = work_id;
      if ( counter = 0 ) then
        insert into work_prop ( id, realm, work, name, mode_type, type, value, required, description )
        values ( work_prop_seq.nextval, realm_id, work_id, wp2_name, wp2_mode_type, wp2_type, wp2_value, wp2_req, wp2_desc );
      end if;
    end if;

    if ( length ( wp3_name ) > 0 ) then
      select count(1) into counter from work_prop t where lower ( t.name ) = lower ( wp3_name ) and t.realm = realm_id and t.work = work_id;
      if ( counter = 0 ) then
        insert into work_prop ( id, realm, work, name, mode_type, type, value, required, description )
        values ( work_prop_seq.nextval, realm_id, work_id, wp3_name, wp3_mode_type, wp3_type, wp3_value, wp3_req, wp3_desc );
      end if;
    end if;

  end loop;
  close cur_realm;

end;
/
commit;